package com.lzmispt.data.resource.service.collectiontool.impl;

package .tmp;

import com.lzmispt.core.dao.database.impl.BaseDaoImpl;
import com.lzmispt.core.entity.coreconst.CoreConst;
import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.core.entity.dto.file.PreUploadParamDTO;
import com.lzmispt.core.entity.po.file.FileUploadRecordPO;
import com.lzmispt.core.entity.vo.file.UploadChunkInfoVO;
import com.lzmispt.core.service.authorize.impl.AuthorizeImpl;
import com.lzmispt.core.service.file.IFileChunkUploadService;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.resource.dao.collectiontool.ICollectionToolDao;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataFileTableDTO;
import com.lzmispt.data.resource.entity.po.collectiontool.FormTemplatePO;
import com.lzmispt.data.resource.entity.po.collectiontool.TempFilePO;
import com.lzmispt.data.resource.entity.vo.collectiontool.FtpVO;
import com.lzmispt.data.resource.entity.vo.collectiontool.TempFileVO;
import com.lzmispt.data.resource.manager.file.FileInfoManager;
import com.lzmispt.data.resource.service.collectiontool.IFileCollection;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.util.HashCodeUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.WorkerId;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Objects;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

@Service
public class IFileCollectionImpl implements IFileCollection {
  private static final String FILE_PATH = System.getProperty("catalina.home") + "/webapps/file/resource";
  
  @Autowired
  BaseDaoImpl baseDao;
  
  @Autowired
  AuthorizeImpl authorize;
  
  @Autowired
  IFileChunkUploadService chunkUploadService;
  
  public ResultEntity<Integer> getChunkSize() {
    return ResultEntity.successMessage(this.chunkUploadService.getChunkSize());
  }
  
  public ResultEntity<UploadChunkInfoVO> preUpload(PreUploadParamDTO param, String platCode, HttpServletRequest request) {
    String source = getIpAddress(request) + "-" + param.getUserId() + "-" + param.getFileName();
    param.setSourceFile(source);
    param.setTempPath(FILE_PATH);
    return this.chunkUploadService.preUpload(param);
  }
  
  public ResultEntity<UploadChunkInfoVO> uploadChunk(Long recordId, MultipartFile uploadFile, String hash, Integer index, String tokenId) {
    FileUploadRecordPO record = (FileUploadRecordPO)this.baseDao.load(recordId, FileUploadRecordPO.class);
    if (record == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getMessage()); 
    if (CoreConst.FileUploadRecordStatus.FINISHED.equals(record.getRecordStatus()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getMessage()); 
    String[] progress = record.getProgress().split(" of ");
    if (!Objects.equals(index, Integer.valueOf(progress[0])))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_REQUIRED_INDEX.getCode()), String.format(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_REQUIRED_INDEX.getMessage(), new Object[] { progress[0] })); 
    File tempChunkFile = new File(record.getTempFilePath() + File.separator + record.getTargetFileName() + "." + index);
    File parentFile = tempChunkFile.getParentFile();
    if (!parentFile.exists())
      parentFile.mkdirs(); 
    try {
      uploadFile.transferTo(tempChunkFile);
    } catch (IOException e) {
      LogHelper.error("保存文件到本地临时路径失败", ExceptionUtil.getStackTraceMsg(e));
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getMessage());
    } 
    if (hash != null) {
      String chunkHash = HashCodeUtil.getHashCode(tempChunkFile, "sha-1");
      if (!Objects.equals(hash, chunkHash))
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_HASH_NOT_EQUAL.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_HASH_NOT_EQUAL.getMessage()); 
    } 
    try (FileInputStream fileInputStream = new FileInputStream(tempChunkFile)) {
      return this.chunkUploadService.uploadChunk(record, fileInputStream, index);
    } catch (Exception e) {
      LogHelper.error("上传切片文件失败", ExceptionUtil.getStackTraceMsg(e));
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getMessage());
    } finally {
      tempChunkFile.delete();
    } 
  }
  
  public ResultEntity<TempFileVO> completeUpload(Long recordId, Long templateId, Long fileTableId, String platCode, String tokenId, Integer dataResourceCategory) {
    FileUploadRecordPO record = (FileUploadRecordPO)this.baseDao.load(recordId, FileUploadRecordPO.class);
    if (record == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getMessage()); 
    if (CoreConst.FileUploadRecordStatus.FINISHED.equals(record.getRecordStatus()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getMessage()); 
    String[] split = record.getProgress().split(" of ");
    if (!Objects.equals(split[0], split[1]))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_FINISHED.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_FINISHED.getMessage()); 
    FtpVO ftpPathVO = getFtpPath(templateId, fileTableId, platCode, dataResourceCategory, tokenId);
    String newFileName = this.chunkUploadService.generateUniqueFileName(record.getSourceFile().split("-", 3)[2]);
    try {
      this.chunkUploadService.moveTempFileToTargetPath(record.getTargetFileName(), ftpPathVO.getFtpPath(), newFileName);
    } catch (IOException e) {
      LogHelper.error("将FTP临时文件移动到目标路径失败", ExceptionUtil.getStackTraceMsg(e));
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPLOAD_TO_FTP_FAILED.getMessage());
    } 
    FileUploadRecordPO update = new FileUploadRecordPO();
    update.setId(recordId);
    update.setTargetFileName(newFileName);
    update.setTargetFilePath(ftpPathVO.getFtpPath());
    update.setRecordStatus(CoreConst.FileUploadRecordStatus.FINISHED);
    update.setUpdateTime(new Date());
    this.baseDao.updateSelective(update);
    record.setTargetFileName(newFileName);
    TempFileVO tempFileVO = saveAndGetTempFileVO(record, ftpPathVO.getFtpPathSave().replace("/+", "/"), platCode, tokenId);
    return ResultEntity.successMessage(tempFileVO);
  }
  
  public ResultEntity<Boolean> stopUpload(Long recordId, String tokenId) {
    FileUploadRecordPO record = (FileUploadRecordPO)this.baseDao.load(recordId, FileUploadRecordPO.class);
    if (record == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_NOT_EXIST.getMessage()); 
    if (record.getRecordStatus().intValue() == 2)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getCode()), StatusCodeEnum.DATA_RESOURCE_CHUNK_UPLOAD_RECORD_ALREADY_FINISHED.getMessage()); 
    FileUploadRecordPO update = new FileUploadRecordPO();
    update.setId(recordId);
    update.setRecordStatus(CoreConst.FileUploadRecordStatus.STOPPED);
    update.setUpdateTime(new Date());
    this.baseDao.updateSelective(update);
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private TempFileVO saveAndGetTempFileVO(FileUploadRecordPO record, String ftpPathSave, String platCode, String tokenId) {
    String oldFileNameAndSuffix = record.getSourceFile().split("-", 3)[2];
    String suffix = oldFileNameAndSuffix.substring(oldFileNameAndSuffix.lastIndexOf(".") + 1);
    String oldFileName = oldFileNameAndSuffix.substring(0, oldFileNameAndSuffix.length() - suffix.length() - 1);
    UserInfoDTO userInfo = this.authorize.getUserInfoByTokenId(tokenId);
    TempFilePO tempFile = new TempFilePO();
    tempFile.setFileId((Long)WorkerId.getNextId(Long.class));
    tempFile.setFtpPath(ftpPathSave);
    tempFile.setFileName(record.getTargetFileName());
    tempFile.setFileExt(suffix);
    tempFile.setFileSize(record.getSourceFileSize());
    tempFile.setOriginalFileName(oldFileName);
    tempFile.setIsTempFile(DCConst.FALSE);
    tempFile.setOrganizationId(userInfo.getOrganizationId());
    tempFile.setUserId(userInfo.getUserId());
    tempFile.setCreateTime(new Date());
    tempFile.setUpdateTime(new Date());
    tempFile.setIsDelete(DCConst.FALSE);
    ((ICollectionToolDao)ServiceUtil.getCollectionToolDao(platCode, ICollectionToolDao.class)).addEntity(tempFile);
    TempFileVO tempFileVO = new TempFileVO();
    BeanUtils.copyProperties(tempFile, tempFileVO);
    tempFileVO.setShowFileName(oldFileNameAndSuffix);
    return tempFileVO;
  }
  
  public static String getIpAddress(HttpServletRequest request) {
    String ip = request.getHeader("x-forwarded-for");
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip))
      ip = request.getHeader("Proxy-Client-IP"); 
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip))
      ip = request.getHeader("WL-Proxy-Client-IP"); 
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip))
      ip = request.getHeader("HTTP_CLIENT_IP"); 
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip))
      ip = request.getHeader("HTTP_X_FORWARDED_FOR"); 
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip))
      ip = request.getRemoteAddr(); 
    return ip;
  }
  
  private FtpVO getFtpPath(Long id, Long fileInfoTableId, String platCode, Integer dataResourceCategory, String tokenId) {
    String ftpPath, ftpPathSave;
    FtpVO ftpVO = new FtpVO();
    FormTemplatePO formTemplate = null;
    if (id != null)
      formTemplate = ((ICollectionToolDao)ServiceUtil.getCollectionToolDao(platCode, ICollectionToolDao.class)).selectFormTemplateById(id); 
    ResAndDataFileTableDTO resAndDataFileTableDTO = FileInfoManager.getFileTableById(fileInfoTableId, platCode, dataResourceCategory, tokenId);
    if (resAndDataFileTableDTO.getFileRootPath() != null && !StringUtils.isEmpty(resAndDataFileTableDTO.getFileRootPath())) {
      ftpPath = resAndDataFileTableDTO.getFileRootPath();
      ftpPathSave = resAndDataFileTableDTO.getFileRootPath();
    } else {
      ftpPath = "DataCenter";
      ftpPathSave = "DataCenter";
    } 
    if (formTemplate != null && !StringUtils.isEmpty(formTemplate.getFilePath())) {
      String templatePath = formTemplate.getFilePath();
      if (!templatePath.startsWith("/"))
        templatePath = "/" + templatePath; 
      ftpPath = ftpPath + File.separator + templatePath;
      ftpPathSave = ftpPathSave + "/" + templatePath;
    } 
    ftpVO.setFtpPath(ftpPath);
    ftpVO.setFtpPathSave(ftpPathSave);
    return ftpVO;
  }
}
